#include "Synthesizer.h"

Synthesizer::Synthesizer( hub::Hub* theHub ) : theHub(theHub){
}

Synthesizer::~Synthesizer(){
}

void Synthesizer::operator()( hub::SrcLocation& src, hub::DestLocation& dest, StateMinimizationAlgorithmIdentificator stateMinimizationAlgorithm, EncodingAlgorithmIdentificator encodingAlgorithm, RealizationAlgorithmIdentificator realizationAlgorithm ){
	src.Find();
	dest.Find();
	
	//source must exist
	if( src.theLib() == 0 ){
		throw CException< std::runtime_error, InconsReason, ToolPosition >(
			InconsReason( "source library does not exist" ),
			ToolPosition( "Synthesizer" )
			);
	}
	
	//source must be Behavioral
	if( src.theArch() == 0 || !src.theArch()->predicate( BehavArch::is ) ){
		throw CException< std::runtime_error, InconsReason, ToolPosition >(
			InconsReason( "source architecture is not behavioral" ),
			ToolPosition( "Synthesizer" )
		);
	}

	// source must be FSM
	const BehavArch* srcArch = src.theArch()->Impl<BehavArch>();
	if( srcArch == 0 || !srcArch->description<FsmDescr>() ){
		throw CException< std::runtime_error, InconsReason, ToolPosition >(
			InconsReason( "source architecture is not FSM" ),
			ToolPosition( "Synthesizer" )
		);
	}

	//destination library must be empty
	if( dest.theLib() != 0 ){
		throw CException< std::runtime_error, InconsReason, ToolPosition >(
			InconsReason( "destination library is already used" ),
			ToolPosition( "Synthesizer" )
		);
	}

	//algorithm identification must not be last marker
	if( lastM == stateMinimizationAlgorithm || lastE == encodingAlgorithm || lastR == realizationAlgorithm ){
		using namespace hub;
		throw CException<std::runtime_error, InconsReason, ToolPosition>( InconsReason ( "Wrong state minimization algorithm - last{MER} is not an algorithm name" ),
			ToolPosition ( "Synthesizer" )
			);
	}

	dest.FindOrCreate();
	
	if( dest.theLib() == 0 || dest.theEntity() == 0 || dest.theArch() == 0 ){
		using namespace hub;
		throw CException<std::runtime_error, InconsReason, ToolPosition>( InconsReason ( "Error missing library, entity or architecture in location" ),
			ToolPosition ( "Synthesizer" )
			);
	}

	//create intermediate fsm destination
	hub::DestLocation destMinimizedFSMLocation( this->theHub );
	destMinimizedFSMLocation = dest.theLib()->name() + "." + dest.theEntity()->name() + ".minimized" + dest.theArch()->name();
	//intermediate destination architecture must be empty
	if( destMinimizedFSMLocation.theArch() != 0 ){
		throw CException< std::runtime_error, InconsReason, ToolPosition >(
			InconsReason( "destination architecture .minimized+name is already used. This name is reserved for internal operation" ),
			ToolPosition( "Synthesizer" )
			);
	}
	destMinimizedFSMLocation.FindOrCreate();
	hub::SrcLocation srcMinimizedFSMLocation( this->theHub );
	srcMinimizedFSMLocation = (std::string) destMinimizedFSMLocation;
	srcMinimizedFSMLocation.Find();

	hub::DestLocation destEncodedFSMLocation( this->theHub );
	destEncodedFSMLocation = dest.theLib()->name() + "." + dest.theEntity()->name() + ".encoded" + dest.theArch()->name();
	//intermediate destination architecture must be empty
	if( destEncodedFSMLocation.theArch() != 0 ){
		throw CException< std::runtime_error, InconsReason, ToolPosition >(
			InconsReason( "destination architecture .encoded+name is already used. This name is reserved for internal operation" ),
			ToolPosition( "Synthesizer" )
			);
	}
	destEncodedFSMLocation.FindOrCreate();
	hub::SrcLocation srcEncodedFSMLocation( this->theHub );
	srcEncodedFSMLocation = (std::string) destEncodedFSMLocation;
	srcEncodedFSMLocation.Find();

	


	infStream << "	Minimizing states with '" << stateMinimizationAlgorithm.getName() <<"' algorithm"<< std::endl;
	StateMinimizer stateMinimizer( this->theHub );
	stateMinimizer( src, destMinimizedFSMLocation, stateMinimizationAlgorithm );

	infStream << "	Encoding with '" << encodingAlgorithm.getName() <<"' algorithm"<< std::endl;
	FsmCode fsmcode( this->theHub );
	fsmcode( srcMinimizedFSMLocation, destEncodedFSMLocation, encodingAlgorithm );

	infStream << "	Realization with '" << realizationAlgorithm.getName() <<"' algorithm"<< std::endl;
	Realizer realizer( this->theHub );
	realizer( srcEncodedFSMLocation, dest, realizationAlgorithm );

}
